precision highp float;


#my_include "./projections.inc.glsl"

//xy - line center position
//zw - line center is ofsetted by this normal vector
attribute vec4 POSITION;

attribute vec4 LINE_INFO;
attribute vec2 SIGN_DISTANCE;
attribute vec3 TEXINFO;


uniform vec4 en_posWorld;
uniform vec3 en_depthZoomScaleRepeat;
uniform float r;

varying float signDist;
varying float startDistance;
varying vec3 posScale;
varying vec2 a;
varying vec2 b;
varying vec3 texInfo;



void main()
{
	signDist = SIGN_DISTANCE.x;
	startDistance = SIGN_DISTANCE.y;

	//for dashing info
	a = LINE_INFO.xy;
	b = LINE_INFO.zw;
	texInfo = TEXINFO;

	posScale.xy = POSITION.xy + POSITION.zw * en_depthZoomScaleRepeat.y;
	posScale.z = en_depthZoomScaleRepeat.z;

	//map range from 0 - 512 to [en_posWorld.xz, en_posWorld.yw]
	//we want to map "position" itself and also "position + normal"
	//that creates offset point
	vec2 rangeScale = (en_posWorld.yw - en_posWorld.xz) * vec2(1.0 / 512.0);

	vec2 wpCenter = en_posWorld.xz + POSITION.xy * rangeScale;
	vec2 wpCenterOffset = POSITION.zw * rangeScale;

	//create inverse projection
	wpCenter.y = 1.0 - wpCenter.y;
	vec2 gpsCenter = (wpCenter.xy + mercatorPrecompInv.xz) * mercatorPrecompInv.yw;
	vec2 gpsCorner = gpsCenter + vec2(1.0, -1.0) * wpCenterOffset * mercatorPrecompInv.yw;

	vec2 tmp = vec2(gpsCenter.y, gpsCorner.y);
	tmp = 2.0 * atan(exp(tmp)) - PI_2;

	gpsCenter.y = tmp.x;
	gpsCorner.y = tmp.y;

	//trial-error created 0.75 constant
	//and exp scale of GPS latitude
	posScale.z /= exp(abs(gpsCenter.y * 0.75));


	vec3 posWorldCorner = ConvertSphericalToCartesian(gpsCorner);
	vec3 posWorldCenter = ConvertSphericalToCartesian(gpsCenter);

	vec3 n = normalize(posWorldCorner - posWorldCenter);
	vec3 posWorld = posWorldCenter + en_depthZoomScaleRepeat.y * n;

	gl_Position = CalcPositionInWorld((r + (1.0 - en_depthZoomScaleRepeat.x)) * posWorld);
	gl_Position.w = 1.0;

}
