//====================================================
// Constants
//====================================================
#define PI 3.1415926535
#define PI2 6.283185307
#define PI_2 1.57079632679
#define PI_4 0.78539816339
#define EXP 2.71828182846

//====================================================
// Projection variables
//====================================================

//-wPadding + wAr * minPixelOffsetX
//1.0 / wAR
//-h + hPadding - hAr * minPixelOffsetY
//1.0 / -hAR
uniform vec4 mercatorPrecompInv;


//wPadding - minPixelOffsetX * wAr
//wAr
//h - hPdadding + minPixelOffsetY * hAr
//hAr
uniform vec4 mercatorPrecomp;

//wPadding - minPixelOffsetX * wAr
//wAr
//h - hPdadding + minPixelOffsetY * hAr
//hAr
uniform vec4 equirectPrecomp;

//w, h, 1/w, 1/h
uniform vec4 texSize;

//tileW, tileH
uniform vec2 tileFrame;

//w, h, w/h, h/w
uniform vec4 screenSize;

//world * view * projection matrix for 3D
//to convert WorldCoord to ScreenSpace
uniform mat4 mWorldViewProj;


uniform sampler2D lut_sincos;

//====================================================
// Calculations
//====================================================

///<summary> 
/// Convert input point in WorldCoord in Mercator 
/// to GPS coordinate
///</summary>
vec2 MercatorProjectInverse(vec2 p)
{
	vec2 tmp = (p.xy + mercatorPrecompInv.xz) * mercatorPrecompInv.yw;
		
	//MercatorProjectInverseInternal
	tmp.y = 2.0 * atan(exp(tmp.y)) - PI_2;
	
	return tmp;
}

///<summary> 
/// Project GPS to WorldCoord in Equirectangular
///</summary>
vec2 EquirectangularProject(vec2 gps)
{	
	return equirectPrecomp.xz + gps.xy * equirectPrecomp.yw;	
}

///<summary> 
/// Convert WorldCoord in Mercator to GPS
///</summary>
vec2 GetCoordinateFromMercatorWorldPos(vec2 worldPos)
{
	// clamping to compensate "world" repeating		
	vec2 pPos = worldPos;
	pPos.y = 1.0 - pPos.y;

	return MercatorProjectInverse(pPos);
}

///<summary> 
/// Convert WorldCoord in Mercator to interval [0, 1] that is
/// in Equirectangular projection
///</summary>
vec2 FromEquirectangularToMercator(vec2 worldPos)
{
	vec2 gps = GetCoordinateFromMercatorWorldPos(worldPos);
	vec2 pixel = EquirectangularProject(gps);

    //vypne interpolaci !!!
    //round "pixel" to whole numbers
    //use floor - we have only + numbers, so its OK
    // 3.7 => 4; 3.2 => 3
    //WIN funguje, iOS ne :(
    //vec2 texel = floor(pixel + vec2(0.5)) + vec2(0.5);
	
	vec2 texel = pixel;// +vec2(0.50, 0.50);
    
	//border check at the end - clamp to last pixel if it is out of tile
	texel = clamp(texel, vec2(0.0), tileFrame);
	
    texel *= texSize.zw;
      
	return texel;
} 


///<summary> 
/// Convert spherical coordinates (gps.x = lon, gps.y = lat)
/// to cartesian coordinates [x, y, z] in Left-handed system
/// Expects unit sphere
/// Source: https://vvvv.org/blog/polar-spherical-and-geographic-coordinates
///</summary>
vec3 ConvertSphericalToCartesian(vec2 gps)
{
/*
	vec2 lon = vec2(gps.x / PI2);
	vec2 lat = vec2(gps.y / PI2);

	vec2 sinCosLon = texture2D(lut_sincos, lon).xy;
	vec2 sinCosLat = texture2D(lut_sincos, lat).xy;
	
	//vec2 sinCosLon = vec2(sin(lon.x), cos(lon.y));
	//vec2 sinCosLat = vec2(sin(lat.x), cos(lat.y));

	//LH system
	vec3 pos;
	pos.x = sinCosLat.y * sinCosLon.x;
	pos.y = sinCosLat.x;
	pos.z = -sinCosLat.y * sinCosLon.y;

	return pos;
*/
	

	vec2 cosXY = cos(gps.xy);
    vec2 sinXY = sin(gps.xy);
    
	//LH system
	vec3 pos;
    pos.x =  cosXY.y * sinXY.x;
    pos.y =  sinXY.y;
    pos.z = -cosXY.y * cosXY.x; //perspective (LH)
	//pos.z *= -1.0; //ortho

	//RH system
	//vec3 pos;
	//pos.x = cosXY.y * cosXY.x;
	//pos.y = cosXY.y * sinXY.x;
	//pos.z = sinXY.y;

	return pos;
	
}

///<summary> 
/// Convert WorldCoord to ScreenSpace projection system
///</summary>
vec4 CalcPositionInWorld(vec3 vPos)
{	
	return vec4(vPos.xyz, 1.0) * mWorldViewProj;
}
