precision highp float;

__USER_DEFINES__

#define TEX2D(pos) texture2D(en_texModel, pos)

#my_include "./projections.inc.glsl"

//======================================================================
//Skeleton for ventusky methods with 1 parametr
//only one input texture is used
// (basically all except wind)
//======================================================================


varying vec2 vTexCoord; //pro projekce


uniform vec4 minMaxStepOpacity;

uniform sampler2D en_texModel;
uniform sampler2D colorMap;

uniform float enableInterpolation;

uniform vec4 texCoordAtlas;

float ConvertPackedValue(__CONVERT_PACKED_VALUE_PARAM__) __CONVERT_PACKED_VALUE_FUNCTION__


vec4 GetUnpackedColor(float val)
{
	//problem.. tohle jde jen pro data zaokrouhleny na cela cisla :(
	//val = floor(val + 0.5);

	float intervalLength = (minMaxStepOpacity.y - minMaxStepOpacity.x);
	float tx = (val - minMaxStepOpacity.x) / intervalLength;
    
	return texture2D(colorMap, vec2(tx, 0.5));
}

#my_include "ventusky/ventusky_layer_interpolations.ps.glsl"


float UnpackValue_BicubicInterpolation_LinearTexValues(vec2 iTc)
{
	vec2 f = vec2(0.0);
			
	vec2 tc0 = vec2(0.0);
	vec2 tc1 = vec2(0.0);
	vec2 tc2 = vec2(0.0);
	vec2 tc3 = vec2(0.0);

    vec4 wx = vec4(0.0);
    vec4 wy = vec4(0.0);
    
    BicubicTexCoord(iTc, f, tc0, tc1, tc2, tc3);
    BicubicWeights(f, wx, wy);

	
	float f00 = TEX2D(vec2(tc0.x, tc0.y)).r;
	float f10 = TEX2D(vec2(tc1.x, tc0.y)).r;
	float f20 = TEX2D(vec2(tc2.x, tc0.y)).r;
	float f30 = TEX2D(vec2(tc3.x, tc0.y)).r;

	float f01 = TEX2D(vec2(tc0.x, tc1.y)).r;
	float f11 = TEX2D(vec2(tc1.x, tc1.y)).r;
	float f21 = TEX2D(vec2(tc2.x, tc1.y)).r;
	float f31 = TEX2D(vec2(tc3.x, tc1.y)).r;

	float f02 = TEX2D(vec2(tc0.x, tc2.y)).r;
	float f12 = TEX2D(vec2(tc1.x, tc2.y)).r;
	float f22 = TEX2D(vec2(tc2.x, tc2.y)).r;
	float f32 = TEX2D(vec2(tc3.x, tc2.y)).r;

	float f03 = TEX2D(vec2(tc0.x, tc3.y)).r;
	float f13 = TEX2D(vec2(tc1.x, tc3.y)).r;
	float f23 = TEX2D(vec2(tc2.x, tc3.y)).r;
	float f33 = TEX2D(vec2(tc3.x, tc3.y)).r;
    
    vec4 cf0 = vec4(f00, f10, f20, f30);
    vec4 cf1 = vec4(f01, f11, f21, f31);
    vec4 cf2 = vec4(f02, f12, f22, f32);
    vec4 cf3 = vec4(f03, f13, f23, f33);
    
    vec4 tmp = vec4(dot(cf0, wx), dot(cf1, wx), dot(cf2, wx), dot(cf3, wx));
    
    return (1.0 / 36.0) * dot(wy, tmp);
}


void main()
{		
	vec2 texCoord = vTexCoord;// *texCoordAtlas.zw + texCoordAtlas.xy;
	texCoord += texCoordAtlas.xy;
	
	//gl_FragColor = TEX2D(texCoord) * 2.0;
	//gl_FragColor.a = 0.5;
	//return;

	
	float unpackedValue = 0.0;

	if (enableInterpolation < 0.5)
	{
        float packedValue = TEX2D(texCoord).r;
		unpackedValue = ConvertPackedValue(packedValue * 255.0);
	}
	else 
	{
#ifdef NON_LINEAR
	//for non-linear convert function, interpolation 
	//cannot be done on "texture" values, but must be done
	//on "unpacked" values
	//in this case, we must run interpolation manually

	#ifdef USE_BICUBIC
        #ifdef USE_LOW_QUALITY			
            unpackedValue = UnpackValue_BilinearInterpolation_NonLinearTexValues(texCoord);
        #else
			unpackedValue = UnpackValue_BicubicInterpolation_NonLinearTexValues(texCoord);
        #endif
	#else
		unpackedValue = UnpackValue_BilinearInterpolation_NonLinearTexValues(texCoord);
	#endif
    

#else
	#ifdef USE_BICUBIC
		#ifdef USE_LOW_QUALITY
			float packedValue = TEX2D(texCoord).r;
		#else
			float packedValue = UnpackValue_BicubicInterpolation_LinearTexValues(texCoord);
		#endif
	#else
		float packedValue = TEX2D(texCoord).r;
	#endif
        unpackedValue = ConvertPackedValue(packedValue * 255.0);
#endif
	}		
    
	vec4 color = GetUnpackedColor(unpackedValue);	
	gl_FragColor = color;
	gl_FragColor.a *= minMaxStepOpacity.w;	//*= multiply opacity from "color map" with "user defined"

	//gl_FragColor.rg = texCoord;
	//gl_FragColor.b = 1.0;
	//gl_FragColor.a = 1.0;

	//DEBUG
    //if ((vTexCoord.x < 0.01) || (vTexCoord.x > 0.99)) gl_FragColor.rgb = vec3(1,1,0);
    //if ((vTexCoord.y < 0.01) || (vTexCoord.y > 0.99)) gl_FragColor.rgb = vec3(1,1,0);
    
}
