precision highp float;



#define TEX2D(pos) texture2D(en_texModel, pos)

//======================================================================
//Skeleton for ventusky methods with 1 parametr
//only one input texture is used
// (basically all except wind)
//======================================================================


varying vec2 vTexCoord; //pro projekce
varying vec2 vWorldPos; //pro projekce
varying vec2 gps;


uniform vec4 minMaxStepOpacity;

uniform sampler2D en_texModel;
uniform sampler2D colorMap;


uniform sampler2D hatchMap;
uniform sampler2D stormMap;
uniform sampler2D freezeMap;
uniform vec3 stormSnowFreezeScale;

//uniform vec2 texSize;
uniform float enableInterpolation;

uniform vec4 texCoordAtlas;

float ConvertPackedValue(float b) 
{   
    b = b - 10.0;
	
	//https://stackoverflow.com/questions/47597588/glsl-optimize-if-else

	float s = 0.0;    
    if (b > 240.0) s = 100.0 * b - 23220.0;
    else if (b > 90.0) s = 5.0 * b - 370.0;
    else if (b >= 10.0) s = b - 10.0;
    else if (b > 0.0) s = b * 0.1;
      			    
	return s;
}

#my_include "./projections.inc.glsl"
#my_include "./ventusky/ventusky_layer_interpolations.ps.glsl"

vec4 GetUnpackedColor(float val)
{			
    float tx = (val - minMaxStepOpacity.x) / (minMaxStepOpacity.y - minMaxStepOpacity.x);
	return texture2D(colorMap, vec2(tx, 0.5));
}

//================================================================================

vec4 PrepareBilinearForTypes(vec2 iTc, out vec2 f)
{	
	vec2 tc0 = vec2(0.0);
	vec2 tc1 = vec2(0.0);
	vec2 tc2 = vec2(0.0);
	vec2 tc3 = vec2(0.0);


	BilinearTexCoord(iTc, f, tc0, tc1, tc2, tc3);

	vec4 val = vec4(0.0);
	
#ifdef GLES_2	
	val.x = TEX2D(tc0).a;
	val.y = TEX2D(tc1).a;
	val.z = TEX2D(tc2).a;
	val.w = TEX2D(tc3).a;
#else	
	val.x = TEX2D(tc0).g;
	val.y = TEX2D(tc1).g;
	val.z = TEX2D(tc2).g;
	val.w = TEX2D(tc3).g;
#endif



	return val * 255.0;
}


float BilinearInterpolationForType(vec4 unpackedValue, vec2 f, float type)
{	
	vec4 tmp = abs(unpackedValue - vec4(type));
	unpackedValue = vec4(lessThan(tmp, vec4(1.0))) * vec4(type);
	
	float tA = mix(unpackedValue.x, unpackedValue.y, f.x);
	float tB = mix(unpackedValue.z, unpackedValue.w, f.x);
	return mix(tA, tB, f.y);	
}


float BilinearInterpolation_Storm(vec4 unpackedValue, vec2 f)
{
	return BilinearInterpolationForType(unpackedValue, f, 40.0);
}
 
float BilinearInterpolation_FreezeRain(vec4 unpackedValue, vec2 f)
{
	return BilinearInterpolationForType(unpackedValue, f, 50.0);
}

float BilinearInterpolation_Snow(vec4 unpackedValue, vec2 f)
{
	//if value is 20 (= convective snow) - replace it with 30 (= snow)
	vec4 tmp = abs(unpackedValue - vec4(20.0));
	
	if (tmp.x < 1.0) unpackedValue.x = 30.0;
	if (tmp.y < 1.0) unpackedValue.y = 30.0;
	if (tmp.z < 1.0) unpackedValue.z = 30.0;
	if (tmp.w < 1.0) unpackedValue.w = 30.0;

	return BilinearInterpolationForType(unpackedValue, f, 30.0);	
}

//================================================================================

float PrepareNoInterpolationForTypes(vec2 iTc)
{
#ifdef GLES_2	
	return TEX2D(iTc).a * 255.0;
#else	
	return TEX2D(iTc).g * 255.0;
#endif
}


float NoInterpolation_Storm(float unpackedVal)
{
	float tmpStorm = abs(unpackedVal - 40.0);
	return (tmpStorm < 1.0) ? 40.0 : 0.0;
}

float NoInterpolation_FreezeRain(float unpackedVal)
{
	float tmpFreezeRain = abs(unpackedVal - 50.0);
	return (tmpFreezeRain < 1.0) ? 50.0 : 0.0;
}


//================================================================================

float MapRange(float fromMin, float fromMax, float toMin, float toMax, float s)
{
	return toMin + (s - fromMin) * (toMax - toMin) / (fromMax - fromMin);
}


float RoundToNearest(float number, float multiple)
{
	return ceil(number / multiple) * multiple;	
}

#ifdef GLOBE_MODE
//https://gamedev.stackexchange.com/questions/184388/fragment-shader-map-dot-texture-repeatedly-over-the-sphere/
vec2 CalcGlobeTextureRepeat(vec2 gps, float density)
{
    vec2 longLat = gps;
    longLat.x = MapRange(-PI, PI, -1.0, 1.0, gps.x);
    longLat.y = MapRange(-PI_2, PI_2, -0.5, 0.5, gps.y);
    
    // Count how many rows we can fit top to bottom at our selected density.
    float rowCount = ceil(density);
    // If it's odd, the 0th row sits at the equator. Otherwise, we need to shift it half a row.
    float even = mod(rowCount + 1.0, 2.0);

    // Scale and rount our latitude to find which row this fragment is in.
    float y = longLat.y * rowCount;
    float row = floor(y - 0.5 * even + 0.5);

    // Compute the radius around the sphere along the center of this row.
    float rowRadians = (row + 0.5 * even) * PI / rowCount;
    float rowRadius = cos(rowRadians);
    
    /*
    vec2 cosXY = cos(gps.xy);
    vec2 sinXY = sin(gps.xy);
    
    //LH system
    vec3 pos;
    pos.x =  cosXY.y * sinXY.x;
    pos.y =  sinXY.y;
    pos.z = -cosXY.y * cosXY.x; //perspective (LH)
    //pos.z *= -1.0; //ortho

    //vec3 pos = ConvertSphericalToCartesian(gps);
    vec3 s = normalize(pos);
    rowRadius = 1.0 - (sign(rowRadians) * s.z);
    */
    
    //not precise but without cos
    //rowRadius = sign(rowRadians) * MapRange(0.0, PI_2, 1.0, 0.0, abs(rowRadians));
    
    // Round this to get the number of texture repeats horizontally.
    float rowRepeats = floor(rowRadius * density * 2.0 + 0.5);

    // Make our texture coordinates.
    vec2 uv;
    uv.x = longLat.x * rowRepeats / 2.0;
    uv.y = y - row + (1.0 - even) * 0.5;
    
    return uv;
}

#endif

void main()
{
	vec2 texCoord = vTexCoord;// *texCoordAtlas.zw + texCoordAtlas.xy;
	texCoord += texCoordAtlas.xy;

	float unpackedValue = 0.0;
		
	if (enableInterpolation < 0.5)
	{
        float packedValue = TEX2D(texCoord).r;
		unpackedValue = ConvertPackedValue(packedValue * 255.0);
	}
	else 
	{        
		#ifdef USE_LOW_QUALITY
			unpackedValue = UnpackValue_BilinearInterpolation_NonLinearTexValues(texCoord);
		#else			
			unpackedValue = UnpackValue_BicubicInterpolation_NonLinearTexValues(texCoord);
		#endif    
	}
    

	
	vec2 f = vec2(0.0);
	vec4 unpackedTypeLin = PrepareBilinearForTypes(texCoord, f);
	float unpackedTypeNon = PrepareNoInterpolationForTypes(texCoord);

	float typeSnow = BilinearInterpolation_Snow(unpackedTypeLin, f);		
	float typeStorm = NoInterpolation_Storm(unpackedTypeNon);		
	float typeFreezeRain = NoInterpolation_FreezeRain(unpackedTypeNon);

    
    vec4 stormColor = vec4(0.0);	
	vec4 snowColor = vec4(0.0);
	vec4 freezeRainColor = vec4(0.0);

	//only one of these conditions can be valid for a single pixel

	if ((typeStorm > 20.0) && (unpackedValue > 0.1))
	{		
#ifdef GLOBE_MODE
        vec2 uv = CalcGlobeTextureRepeat(gps, stormSnowFreezeScale.x * 0.75);
        float stormVal = texture2D(stormMap, uv).r;
#else
        float stormVal = texture2D(stormMap, stormSnowFreezeScale.x * vWorldPos.xy).r;
#endif
        
		stormColor = vec4(0.82, 0.04, 0.04, 1.0) * stormVal;
	}	 


	if ((typeFreezeRain > 20.0) && (unpackedValue > 0.1))
	{
#ifdef GLOBE_MODE
		vec2 uv = CalcGlobeTextureRepeat(gps, stormSnowFreezeScale.z * 0.75);
		float freezeRainVal = texture2D(freezeMap, uv).r;
#else
		float freezeRainVal = texture2D(freezeMap, stormSnowFreezeScale.z * vWorldPos.xy).r;
#endif

		freezeRainColor = vec4(1.0, 1.0, 1.0, 1.0) * freezeRainVal;
	}
	

	if (typeSnow > 15.0)
	{
#ifdef GLOBE_MODE
        vec2 uv = CalcGlobeTextureRepeat(gps, stormSnowFreezeScale.y * 0.75);
		snowColor = texture2D(hatchMap, uv).rrrr;
#else
        snowColor = texture2D(hatchMap, stormSnowFreezeScale.y * vWorldPos.xy).rrrr;
#endif
	}

	vec4 color = GetUnpackedColor(unpackedValue);
	
	color = snowColor + color * (1.0 - snowColor.a);
	color = stormColor + color * (1.0 - stormColor.a);
	color = freezeRainColor + color * (1.0 - freezeRainColor.a);

	gl_FragColor = color;	
	gl_FragColor.a *= minMaxStepOpacity.w;	//*= multiply opacity from "color map" with "user defined"		

	//DEBUG
	//if ((reprojTx.x < 0.001) || (reprojTx.x > 0.999)) gl_FragColor.rgb = vec3(1.0);
	//if ((reprojTx.y < 0.001) || (reprojTx.y > 0.999)) gl_FragColor.rgb = vec3(1.0);
	//if ((reprojTx.x < 0.001) || (reprojTx.x > 0.999)) gl_FragColor.rgb *= vec3(0.8);
	//if ((reprojTx.y < 0.001) || (reprojTx.y > 0.999)) gl_FragColor.rgb *= vec3(0.8);
}
