precision highp float;


#define TEX2D(pos) texture2D(en_texModel, pos)


varying vec2 vTexCoord; //pro projekce
varying vec2 vWorldPos; //pro projekce


uniform vec4 minMaxStepOpacity;

uniform sampler2D en_texModel;

#my_include "./projections.inc.glsl"


// from http://www.java-gaming.org/index.php?topic=35123.0
vec4 cubic(float v) {
	vec4 n = vec4(1.0, 2.0, 3.0, 4.0) - v;
	vec4 s = n * n * n;
	float x = s.x;
	float y = s.y - 4.0 * s.x;
	float z = s.z - 4.0 * s.y + 6.0 * s.x;
	float w = 6.0 - x - y - z;
	return vec4(x, y, z, w) * (1.0 / 6.0);
}

vec4 textureBicubic(sampler2D sampler, vec2 texCoords) {

	//vec2 texSize = textureSize(sampler, 0);
	vec2 invTexSize = 1.0 / texSize.xy;

	texCoords = texCoords * texSize.xy - 0.5;


	vec2 fxy = fract(texCoords);
	texCoords -= fxy;

	vec4 xcubic = cubic(fxy.x);
	vec4 ycubic = cubic(fxy.y);

	vec4 c = texCoords.xxyy + vec2(-0.5, +1.5).xyxy;

	vec4 s = vec4(xcubic.xz + xcubic.yw, ycubic.xz + ycubic.yw);
	vec4 offset = c + vec4(xcubic.yw, ycubic.yw) / s;

	offset *= invTexSize.xxyy;

	vec4 sample0 = texture2D(sampler, offset.xz);
	vec4 sample1 = texture2D(sampler, offset.yz);
	vec4 sample2 = texture2D(sampler, offset.xw);
	vec4 sample3 = texture2D(sampler, offset.yw);

	float sx = s.x / (s.x + s.y);
	float sy = s.z / (s.z + s.w);

	return mix(
		mix(sample3, sample2, sx), mix(sample1, sample0, sx)
		, sy);
}


void main()
{		
	
	vec4 color = TEX2D(vTexCoord).rgba;
	//vec4 color = textureBicubic(en_texModel, reprojTx);

	//color.xy = reprojTx;
	//color.z = 1.0;

	gl_FragColor = color;	
	gl_FragColor.a *= minMaxStepOpacity.w;	//*= multiply opacity from "color map" with "user defined"		

	//DEBUG
	//if ((reprojTx.x < 0.01) || (reprojTx.x > 0.99)) gl_FragColor.rgb = vec3(1.0, 0.0, 0.0);
	//if ((reprojTx.y < 0.01) || (reprojTx.y > 0.99)) gl_FragColor.rgb = vec3(1.0, 0.0, 0.0);
	//if ((reprojTx.x < 0.001) || (reprojTx.x > 0.999)) gl_FragColor.rgb *= vec3(0.8);
	//if ((reprojTx.y < 0.001) || (reprojTx.y > 0.999)) gl_FragColor.rgb *= vec3(0.8);
}
